//
//  BookTableViewController.swift
//  BookList
//
//  Created by Natsuko Nishikata on 2015/12/09.
//  Copyright © 2015年 Natsuko Nishikata. All rights reserved.
//

import UIKit
import CoreData

class BookTableViewController: UITableViewController {
    
    let coreDataStack = CoreDataStack()
    var books = [Book]()
    
    // フェッチ要求
    lazy var fetchRequest:NSFetchRequest = {
        let fetchRequest = NSFetchRequest(entityName: "Book")
        
        // ソート順を指定
        let sortDescriptor = NSSortDescriptor(key: "registeredDate", ascending: false)
        fetchRequest.sortDescriptors = [sortDescriptor]
        
        return fetchRequest
    }()

    override func viewDidLoad() {
        super.viewDidLoad()
        
        navigationItem.leftBarButtonItem = editButtonItem()
        
        // Self-Sizingセルのための設定
        tableView.rowHeight = UITableViewAutomaticDimension
        tableView.estimatedRowHeight = 56.0
        
        // 永続ストアの追加
        userInteractionEnabled(false)
        coreDataStack.addPersistentStoreWithCompletionHandler() {
            // ストアの追加が完了
            dispatch_async(dispatch_get_main_queue()) {
                self.userInteractionEnabled(true)
                // フェッチ
                self.fetchBooks()
            }
        }
    }

    override func didReceiveMemoryWarning() {
        super.didReceiveMemoryWarning()
        // Dispose of any resources that can be recreated.
    }
    
    override func setEditing(editing: Bool, animated: Bool) {
        super.setEditing(editing, animated: animated)
        navigationItem.rightBarButtonItem?.enabled = !editing
    }


    // MARK: - Table view data source
    
    // テーブルビューのセクション数
    override func numberOfSectionsInTableView(tableView: UITableView) -> Int {
        return 1
    }

    // セクションsectionに含まれる行数（必須）
    override func tableView(tableView: UITableView, numberOfRowsInSection section: Int) -> Int {
        return books.count
    }
    
    // セルの設定（必須）
    override func tableView(tableView: UITableView, cellForRowAtIndexPath indexPath: NSIndexPath) -> UITableViewCell {
            
        // "BookCell"はStoryboardで設定済み
        let cell = tableView.dequeueReusableCellWithIdentifier("BookCell",
            forIndexPath: indexPath) as! BookTableViewCell
        
        // セルを設定
        let book = books[indexPath.row]
        cell.configureWithBook(book)
        
        return cell
    }

    // 編集可能かどうかを示す
    override func tableView(tableView: UITableView, canEditRowAtIndexPath indexPath: NSIndexPath) -> Bool {
        return true
    }
    
    // 削除時の処理を実装する
    override func tableView(tableView: UITableView, commitEditingStyle editingStyle: UITableViewCellEditingStyle, forRowAtIndexPath indexPath: NSIndexPath) {
        
        if editingStyle == .Delete {
            
            // 削除対象となるBookオブジェクト
            let book = books[indexPath.row]
            // 管理オブジェクトコンテキストから削除
            coreDataStack.context.deleteObject(book)
            try! coreDataStack.saveContext()
            
            // データソースから削除
            books.removeAtIndex(indexPath.row)
            // テーブルビューを更新
            tableView.deleteRowsAtIndexPaths([indexPath], withRowAnimation: .Fade)
        }
    }

    /*
    // MARK: - Navigation

    // In a storyboard-based application, you will often want to do a little preparation before navigation
    override func prepareForSegue(segue: UIStoryboardSegue, sender: AnyObject?) {
        // Get the new view controller using segue.destinationViewController.
        // Pass the selected object to the new view controller.
    }
    */

    // MARK: - Actions
    
    @IBAction func addBook(sender: UIBarButtonItem) {
        
        // coreDataStack.contextに新しい管理オブジェクトnewBookを追加
        let newBook =
        NSEntityDescription.insertNewObjectForEntityForName("Book",
            inManagedObjectContext: coreDataStack.context) as! Book
        
        newBook.title = "タイトル"  // <--- 仮データをセット
        newBook.author = "著者"    // <--- 仮データをセット
        newBook.url = NSURL(string: "http://www.abc.com") // <--- 仮データをセット
        
        try! coreDataStack.saveContext()
        
        // データソースを更新
        books.insert(newBook, atIndex: 0)
        
        // テーブルビューを更新
        let indexPath = NSIndexPath(forRow: 0, inSection: 0)
        tableView.insertRowsAtIndexPaths([indexPath], withRowAnimation: .Automatic)
    }
    
    @IBAction func segmentChanged(sender: UISegmentedControl) {
        // index 1のセグメントが選択されているときは、欲しい本リストを作る
        let wish = sender.selectedSegmentIndex == 1
        if wish {
            // wishがtrueのBookだけをフェッチするための条件指定
            let predicate = NSPredicate(format: "wish == true")
            fetchRequest.predicate = predicate
        } else {
            // 条件指定を解除
            fetchRequest.predicate = nil
        }
        fetchBooks()
    }
    
    // MARK: - Core Data
    private func fetchBooks() {
        do {
            // フェッチ結果をデータソースとして保持
            books = try coreDataStack.context.executeFetchRequest(fetchRequest) as! [Book]
        } catch let error as NSError {
            fatalError("\(error)")
        }
        
        // テーブルビューをリロード
        tableView.reloadData()
    }
    
    
    // MARK: - private methods
    private func userInteractionEnabled(enabled:Bool) {
        self.navigationItem.rightBarButtonItem?.enabled = enabled
        self.navigationItem.leftBarButtonItem?.enabled = enabled
        (self.navigationItem.titleView as! UISegmentedControl).enabled = enabled
    }

}

class BookTableViewCell: UITableViewCell {
    @IBOutlet weak var titleLabel: UILabel!
    @IBOutlet weak var authorLabel: UILabel!
    @IBOutlet weak var wishLabel: UILabel!
    @IBOutlet weak var dateLabel: UILabel!
    
    lazy var dateFormatter: NSDateFormatter = {
        let dateFormatter = NSDateFormatter()
        dateFormatter.dateStyle = .ShortStyle
        dateFormatter.timeStyle = .MediumStyle
        return dateFormatter
    }()
    
    func configureWithBook(book:Book) {
        titleLabel.text = book.title
        authorLabel.text = book.author
        wishLabel.hidden = !(book.wish!.boolValue)
        dateLabel.text =
            dateFormatter.stringFromDate(book.registeredDate!)
    }
    
}
