//
//  ShelfTableViewController.swift
//  BookList
//
//  Created by Natsuko Nishikata on 2015/12/15.
//  Copyright © 2015年 Natsuko Nishikata. All rights reserved.
//

import UIKit
import CoreData

protocol ShelfTableViewControllerDelegate: class {
    func shelfTableViewController(controller: ShelfTableViewController,
        didSelectShelfWithObjectID objectID:NSManagedObjectID?)
}

class ShelfTableViewController: UITableViewController {
    
    var context:NSManagedObjectContext!
    var shelfs = [Shelf]()
    var singleEdit = false
    
    var selectedObjectID:NSManagedObjectID?
    weak var delegate:ShelfTableViewControllerDelegate?
    
    lazy var fetchRequest: NSFetchRequest = {
        let fetchRequest = NSFetchRequest(entityName: "Shelf")
        
        // ソート順を指定
        let sortDescriptor =
        NSSortDescriptor(key: "displayOrder", ascending: true)
        fetchRequest.sortDescriptors = [sortDescriptor]
        
        return fetchRequest
    }()

    override func viewDidLoad() {
        super.viewDidLoad()
        navigationItem.rightBarButtonItem = editButtonItem()
        tableView.allowsSelectionDuringEditing = true
        
        // Shelfのフェッチ
        fetchShelfs()
        
    }

    override func didReceiveMemoryWarning() {
        super.didReceiveMemoryWarning()
        // Dispose of any resources that can be recreated.
    }
    
    override func setEditing(editing: Bool, animated: Bool) {
        super.setEditing(editing, animated: animated)
        
        if !singleEdit {
            let indexPath = NSIndexPath(forRow: shelfs.count, inSection: 0)
            if editing {
                // 挿入用セルを追加
                tableView.insertRowsAtIndexPaths([indexPath], withRowAnimation: .Fade)
            } else {
                // 挿入用セルを削除
                tableView.deleteRowsAtIndexPaths([indexPath], withRowAnimation: .Fade)
            }
        }
        navigationItem.hidesBackButton = editing
    }

    // MARK: - Table view data source
    // MARK: テーブルビューの表示と選択時の処理
    // セクション数（UITableViewDataSource）
    override func numberOfSectionsInTableView(tableView: UITableView) -> Int {
        return 1
    }
    
    // 行数（UITableViewDataSource）
    override func tableView(tableView: UITableView, numberOfRowsInSection section: Int) -> Int {
        return editing ? shelfs.count + 1 : shelfs.count
    }
    
    // セルの設定（UITableViewDataSource）
    override func tableView(tableView: UITableView, cellForRowAtIndexPath indexPath: NSIndexPath) -> UITableViewCell {
        let cell = tableView.dequeueReusableCellWithIdentifier("ShelfCell", forIndexPath: indexPath)
        
        if indexPath.row < shelfs.count {
            // 通常のShelfセル
            let shelf = shelfs[indexPath.row]
            cell.textLabel?.text = shelf.name
            // セルのアクセサリータイプを設定
            if let selectedID = selectedObjectID {
                cell.accessoryType = (selectedID == shelf.objectID ? .Checkmark : .None)
            } else {
                cell.accessoryType = .None
            }
        } else {
            // 挿入用セル
            cell.textLabel?.text = nil
            cell.accessoryType = .None
        }
        return cell
    }
    
    // セル選択時の処理（UITableViewDelegate）
    override func tableView(tableView: UITableView, didSelectRowAtIndexPath indexPath: NSIndexPath) {
            
        if !tableView.editing { // 通常モード
            tableView.deselectRowAtIndexPath(indexPath, animated: true)
            // 本棚の選択・非選択
            selectShelfAtIndex(indexPath.row)
        } else { // 編集モード
            if indexPath.row < shelfs.count {
                // Shelfの編集
                let shelf = shelfs[indexPath.row]
                presentAlertViewForInputName(shelf.name) {
                    [unowned self] (name:String?) -> Void in
                    self.updateShelfForIndex(indexPath.row, withName: name)
                }
            } else {
                // Shelfの追加
                presentAlertViewForInputName(nil) {
                    [unowned self] (name:String?) -> Void in
                    self.addShelfWithName(name)
                }
            }
        }
    }
    
    // MARK: セルの追加と削除
    // 編集モード確定時の処理（UITableViewDataSource）
    override func tableView(tableView: UITableView, commitEditingStyle editingStyle: UITableViewCellEditingStyle, forRowAtIndexPath indexPath: NSIndexPath) {
        
        if editingStyle == .Delete { // 削除
            deleteShelfAtIndex(indexPath.row)
        } else if editingStyle == .Insert { // 追加
            presentAlertViewForInputName(nil) {
                [unowned self](name:String?) -> Void in
                self.addShelfWithName(name)
            }
        }
    }
    
    // セルが編集可能かどうか（UITableViewDataSource）
    override func tableView(tableView: UITableView, canEditRowAtIndexPath indexPath: NSIndexPath) -> Bool {
        return true
    }
    
    // セルの編集スタイル（UITableViewDelegate）
    override func tableView(tableView: UITableView, editingStyleForRowAtIndexPath indexPath: NSIndexPath) -> UITableViewCellEditingStyle {
        return indexPath.row < shelfs.count ? .Delete : .Insert
    }
    
    // スワイプによる編集の開始（UITableViewDelegate）
    override func tableView(tableView: UITableView, willBeginEditingRowAtIndexPath indexPath: NSIndexPath) {
        singleEdit = true
    }
    
    // スワイプによる編集の終了（UITableViewDelegate）
    override func tableView(tableView: UITableView, didEndEditingRowAtIndexPath indexPath: NSIndexPath) {
        singleEdit = false
    }
    
    // MARK: セルの移動
    
    // 並び替え処理（UITableViewDataSource）
    override func tableView(tableView: UITableView, moveRowAtIndexPath fromIndexPath: NSIndexPath, toIndexPath: NSIndexPath) {
        // データソースの更新
        let shelf = shelfs[fromIndexPath.row]
        shelfs.removeAtIndex(fromIndexPath.row)
        shelfs.insert(shelf, atIndex: toIndexPath.row)
            
        // displayOrderの変更範囲
        var startIndex = fromIndexPath.row
        var endIndex = toIndexPath.row
            
        if fromIndexPath.row > toIndexPath.row {
            startIndex = toIndexPath.row
            endIndex = fromIndexPath.row
        }
        
        // displayOrderの更新
        for i in startIndex...endIndex {
            let obj = shelfs[i]
            obj.displayOrder = i
        }
        
        // 管理オブジェクトコンテキストの保存
        try! context.save()
    }
    
    // 移動可能なセルを返す（UITableViewDataSource）
    override func tableView(tableView: UITableView, canMoveRowAtIndexPath indexPath: NSIndexPath) -> Bool {
        return indexPath.row < self.shelfs.count
    }
    
    // 移動禁止セルの処理（UITableViewDelegate）
    override func tableView(tableView: UITableView, targetIndexPathForMoveFromRowAtIndexPath sourceIndexPath: NSIndexPath, toProposedIndexPath proposedDestinationIndexPath: NSIndexPath) -> NSIndexPath {
        if proposedDestinationIndexPath.row == shelfs.count {
            return sourceIndexPath
        } else {
            return proposedDestinationIndexPath
        }
    }

    // MARK: - Core Data private methods
    private func fetchShelfs() {
        do {
            shelfs = try context.executeFetchRequest(fetchRequest) as! [Shelf]
        } catch let error as NSError {
            fatalError("\(error)")
        }
        
        tableView.reloadSections(NSIndexSet(index: 0), withRowAnimation: .Fade)
    }
    
    private func addShelfWithName(name:String?) {
        
        // 新たなShelfオブジェクトを生成、コンテキストに追加
        let newShelf =
        NSEntityDescription.insertNewObjectForEntityForName("Shelf",
            inManagedObjectContext: context) as! Shelf
        // 名前と並び順を設定
        newShelf.name = name
        newShelf.displayOrder = shelfs.count
        
        // コンテキストを保存
        do {
            try context.save()
            
            // データソースに追加
            shelfs.append(newShelf)
            // テーブルビューの更新
            let indexPath = NSIndexPath(forRow: shelfs.count-1, inSection: 0)
            tableView.insertRowsAtIndexPaths([indexPath], withRowAnimation: .Fade)
        } catch let error as NSError {
            // コンテキストを元に戻す。テーブルビューは変更しない
            context.rollback()
            // エラー処理
            print("\(error)")
        }
    }

    private func deleteShelfAtIndex(index:Int) {
        
        let shelf = shelfs[index]
        
        // コンテキストから削除
        context.deleteObject(shelf)
        // 並び順の付け替え
        for i in index+1..<shelfs.count {
            shelfs[i].displayOrder = i-1
        }
        
        // コンテキストを保存
        do {
            try context.save()
            
            // データソースから削除
            shelfs.removeAtIndex(index)
            // テーブルビューを更新
            let indexPath = NSIndexPath(forRow: index, inSection: 0)
            tableView.deleteRowsAtIndexPaths([indexPath], withRowAnimation: .Fade)
        } catch let error as NSError {

            // コンテキストを元に戻す
            context.rollback()
            // エラー処理
            print("\(error)")
        }
    }
    
    private func updateShelfForIndex(index:Int, withName name:String?) {
        
        let shelf = shelfs[index]
        shelf.name = name
        
        do {
            try context.save()
            
            let indexPath = NSIndexPath(forRow: index, inSection: 0)
            tableView.reloadRowsAtIndexPaths([indexPath], withRowAnimation: .Fade)
            
        } catch let error as NSError {
            context.rollback()
            print("\(error)")
        }
    }
    
    private func presentAlertViewForInputName(name: String?, actionHandler: ((String?)->Void)) {
            
        // 名前入力用のアラートビュー
        let ac = UIAlertController(title: "本棚の追加・編集",
            message: "本棚名を入力してください",
            preferredStyle: .Alert)
        // テキストフィールドを追加
        ac.addTextFieldWithConfigurationHandler {
            (textField:UITextField) -> Void in
            textField.text = name // テキストフィールドの初期値 --- (1)
        }
        let ok = UIAlertAction(title: "OK", style: .Default) {
            (alertAction:UIAlertAction) -> Void in
            
            // テキストフィールドの内容をcompletionHandlerに渡す --- (2)
            let textFields = ac.textFields!
            actionHandler(textFields.first!.text)
        }
        
        let cancel =
        UIAlertAction(title: "Cancel", style: .Cancel, handler: nil)
        
        ac.addAction(ok)
        ac.addAction(cancel)
            
        // アラートビューを表示
        ac.view.setNeedsLayout() // レイアウトに関するエラーを非表示にするための暫定対策（本書P.245参照）
        presentViewController(ac, animated: true, completion: nil)
    }
    
    private func selectShelfAtIndex(index:Int) {
        // 現在選択済みのインデックスの取得
        var oldIndex: Int?
        if let oldID = selectedObjectID {
            // objectIDのみのArrayを取得 --- (1)
            let objectIDs = shelfs.map {$0.objectID}
            // 選択済み本棚のインデックスを取得
            oldIndex = objectIDs.indexOf(oldID)
        }
        
        // selectedObjectIDの更新 --- (2)
        if index != oldIndex {
            // 選択
            let shelf = shelfs[index]
            selectedObjectID = shelf.objectID
        } else {
            // 選択解除
            selectedObjectID = nil
        }
        
        // テーブルビューセルの更新 --- (3)
        if oldIndex != nil {
            // チェックマークをはずす
            let indexPath = NSIndexPath(forRow: oldIndex!, inSection: 0)
            let oldCell = tableView.cellForRowAtIndexPath(indexPath)!
            oldCell.accessoryType = .None
        }
        
        if let _ = selectedObjectID {
            // チェックマークをつける
            let indexPath = NSIndexPath(forRow: index, inSection: 0)
            let newCell = tableView.cellForRowAtIndexPath(indexPath)!
            newCell.accessoryType = .Checkmark
        }
        
        self.delegate?.shelfTableViewController(self, didSelectShelfWithObjectID: selectedObjectID)
    }


}
