//
//  Book.swift
//  BookList
//
//  Created by Natsuko Nishikata on 2015/12/09.
//  Copyright © 2015年 Natsuko Nishikata. All rights reserved.
//

import Foundation
import CoreData

let kBookListErrorDomain = "com.yourcompany.BookList.errorDomain"
enum BookErrorCode:Int {
    case InvalidTitle = 1001
    case NoURL = 1002
}

class Book: NSManagedObject {
    
    override func awakeFromInsert() {
        // 最初に必ずsuperをコール
        super.awakeFromInsert()
        // 内部的な変更なのでプリミティブアクセサを使用
        setPrimitiveValue(NSDate(), forKey: "registeredDate")
    }
    
    override func validateForUpdate() throws {
        // スーパークラスの呼び出し（標準検証）
        try super.validateForUpdate()
        // カスタム検証
        try self.validateUrlAndWish()
    }
    
    override func validateForInsert() throws {
        // スーパークラス呼び出し（標準検証）
        try super.validateForInsert()
        // カスタム検証
        try self.validateUrlAndWish()
    }
    
    override func didChangeValueForKey(key: String) {
        if key == "title" {
            setPrimitiveValue(nil, forKey: "sectionIdentifier")
        }
    }
    
    var recently:NSNumber? {
        let cal = NSCalendar.currentCalendar()
        
        // 1ヶ月前の日付オブジェクト
        let oneMonthAgo =
        cal.dateByAddingUnit(NSCalendarUnit.Month,
            value: -1,
            toDate: NSDate(),
            options:NSCalendarOptions.WrapComponents)!
        
        // 登録日と1ヶ月前の日付を比較し、登録日の方が新しければ「recent」とする
        let date = primitiveValueForKey("registeredDate") as! NSDate
        if cal.compareDate(date,
            toDate: oneMonthAgo,
            toUnitGranularity: NSCalendarUnit.Day)
            == NSComparisonResult.OrderedDescending {
                return true
        } else {
            return false
        }
    }
    
    var sectionIdentifier:String? {
        // プリミティブ値にアクセス --- (1)
        willAccessValueForKey("sectionIdentifier")
        var tmp = primitiveValueForKey("sectionIdentifier") as? String
        didAccessValueForKey("sectionIdentifier")
        
        if tmp == nil { // プリミティブ値がnilの場合
            let title = primitiveValueForKey("title") as? String
            if title != nil && !(title!.isEmpty) {
                tmp = String(title![title!.startIndex])
            } else {
                tmp = ""
            }
            // プリミティブ値をセット --- (2)
            setPrimitiveValue(tmp, forKey: "sectionIdentifier")
        }
        return tmp
    }
    
    var title:String? {
        set {
            willChangeValueForKey("title")
            setPrimitiveValue(newValue, forKey: "title")
            didChangeValueForKey("title")
         
            // titleがセットされたらsectionIdentifierをリセット
            setPrimitiveValue(nil, forKey: "sectionIdentifier")
        }
        get {
            willAccessValueForKey("title")
            let value = primitiveValueForKey("title") as? String
            didAccessValueForKey("title")
            return value
        }
    }
    
    // MARK: - validation
    func validateTitle(value: AutoreleasingUnsafeMutablePointer<AnyObject?>) throws {
       
        if value == nil || value.memory == nil {
            // nilはシステム検証メソッドで検出
            return
        }
        
        let title = value.memory as! String
        
        if title.isEmpty {
            // 長さゼロはシステム検証メソッドで検出
            return
        }
        
        // titleから前後のスペースを削除
        let whitespaces = NSCharacterSet.whitespaceCharacterSet()
        let trimmedTitle = title.stringByTrimmingCharactersInSet(whitespaces)
        if !trimmedTitle.isEmpty {
            // 空白以外の文字が存在する→エラーではない
            return
        }
        
        // エラーの生成と送出
        let userInfo = [NSLocalizedDescriptionKey: "タイトル未入力",
            NSLocalizedRecoverySuggestionErrorKey: "タイトルに空白以外の文字列を入力してください。"]
        throw NSError(domain: kBookListErrorDomain,
            code: BookErrorCode.InvalidTitle.rawValue,
            userInfo: userInfo)
    }
    
    func validateUrlAndWish() throws {
        
        if !wish!.boolValue {
            // wishがfalseのときはURLを検証する必要なし → urlとwishは有効
            return
        }
        
        if let urlString = url?.absoluteString where !urlString.isEmpty {
            // URLの設定あり → urlとwishは有効
            return
        }
        
        let userInfo = [NSLocalizedDescriptionKey: "URL未入力",
            NSLocalizedRecoverySuggestionErrorKey: "「欲しい本」のときはURLの入力が必要です。"]
        
        let error = NSError(domain: kBookListErrorDomain,
            code: BookErrorCode.NoURL.rawValue,
            userInfo: userInfo)
        
        throw error
    }
}
