//
//  BookEditTableViewController.swift
//  BookList
//
//  Created by Natsuko Nishikata on 2015/12/14.
//  Copyright © 2015年 Natsuko Nishikata. All rights reserved.
//

import UIKit
import CoreData

class BookEditTableViewController: UITableViewController, UITextFieldDelegate, UIImagePickerControllerDelegate, UINavigationControllerDelegate, ShelfTableViewControllerDelegate {

    @IBOutlet weak var titleTextField: UITextField!
    @IBOutlet weak var authorTextField: UITextField!
    @IBOutlet weak var urlTextField: UITextField!
    @IBOutlet weak var wishSwitch: UISwitch!
    @IBOutlet weak var registeredDateLabel: UILabel!
    @IBOutlet weak var recentlyLabel: UILabel!
    @IBOutlet weak var photoImageView: UIImageView!
    @IBOutlet weak var shelfNameLabel: UILabel!
    
    var book:Book!
    var coreDataStack:CoreDataStack!
    weak var editingTextField:UITextField?
    
    lazy var dateFormatter: NSDateFormatter = {
        let dateFormatter = NSDateFormatter()
        dateFormatter.dateStyle = .MediumStyle
        dateFormatter.timeStyle = .ShortStyle
        return dateFormatter
    }()
    
    lazy var shelfContext:NSManagedObjectContext = {
        let context = NSManagedObjectContext(concurrencyType: .MainQueueConcurrencyType)
        context.persistentStoreCoordinator = self.coreDataStack.context.persistentStoreCoordinator
        
        let nc = NSNotificationCenter.defaultCenter()
        nc.addObserver(self,
            selector: "shelfContextDidSave:",
            name: NSManagedObjectContextDidSaveNotification,
            object: context)
        
        return context
    }()
    
    deinit {
        NSNotificationCenter.defaultCenter().removeObserver(self)
    }
    
    override func viewDidLoad() {
        super.viewDidLoad()
        
        titleTextField.text = book.title // タイトル
        authorTextField.text = book.author // 著者
        urlTextField.text = book.url?.absoluteString // URL
        wishSwitch.on = book.wish!.boolValue // 欲しい本
        registeredDateLabel.text = dateFormatter.stringFromDate(book.registeredDate!) // 登録日
        
        // 最近の本（recentlyがfalseの場合ラベルを隠す）
        recentlyLabel.hidden = book.recently!.boolValue
        
        // 写真
        if let data = book.photo?.image {
            photoImageView.image = UIImage(data: data)
        }

        // 本棚の名前
        shelfNameLabel.text = book.shelf?.name
        
        // ビューコントローラのタイトル
        if book.title != nil {
            title = "Edit Book"
        } else {
            title = "New Book"
        }
    }

    override func didReceiveMemoryWarning() {
        super.didReceiveMemoryWarning()
        // Dispose of any resources that can be recreated.
    }

    
    // MARK: - UITableViewDelegate
    
    // セル選択時に呼ばれる
    override func tableView(tableView: UITableView, didSelectRowAtIndexPath indexPath: NSIndexPath) {
        // セクション1（画像表示セル）のときのみ実行
        if indexPath.section == 1 {
            let ipc = UIImagePickerController()
            ipc.delegate = self
            self.presentViewController(ipc, animated: true, completion: nil)
        }
    }
    

    // MARK: - Navigation
    override func prepareForSegue(segue: UIStoryboardSegue, sender: AnyObject?) {
        if segue.identifier == "PickShelf" {
            let vc = segue.destinationViewController as! ShelfTableViewController
            vc.context = shelfContext
            vc.selectedObjectID = book.shelf?.objectID
            vc.delegate = self
        }
    }
    
    // MARK: - Notification
    func shelfContextDidSave(notification: NSNotification) {

        // コンテキスト上のオブジェクトをマージ
        let context = coreDataStack.context
        context.mergeChangesFromContextDidSaveNotification(notification)
        
        // UIの更新
        shelfNameLabel.text = book.shelf?.name
    }
    
    // MARK: - Actions
    @IBAction func done(sender: UIBarButtonItem) {
        
        // テキストフィールドの編集を終わらせる→コンテキストに反映する --- (1)
        editingTextField?.resignFirstResponder()
        
        do {
            try coreDataStack.saveContext() // 保存 --- (2)
            // 編集画面を閉じる --- (3)
            dismissViewControllerAnimated(true, completion: nil)
        } catch let error as NSError {
            // エラー表示 --- (4)
            let ac = UIAlertController(title: error.localizedDescription,
                message: error.localizedRecoverySuggestion,
                preferredStyle: .Alert)
            let ok = UIAlertAction(title: "OK",
                style: .Default,
                handler: nil)
            ac.addAction(ok)
            presentViewController(ac, animated: true, completion: nil)
        }
    }
    
    @IBAction func cancel(sender: UIBarButtonItem) {
        // テキストフィールドの編集を終わらせる→コンテキストに反映する
        editingTextField?.resignFirstResponder()
        
        // 削除済みオブジェクトの保持（ここではすべてShelfオブジェクト）
        let deletedObjects = coreDataStack.context.deletedObjects
        
        // コンテキストのロールバック→前回保存またはフェッチされた状態に戻す
        coreDataStack.context.rollback()
        // ロールバックにより復活した削除済みオブジェクトを削除しコンテキストを保存
        for object in deletedObjects {
            coreDataStack.context.deleteObject(object)
        }
        try! coreDataStack.saveContext()
        
        self.dismissViewControllerAnimated(true, completion: nil)
    }
    
    @IBAction func wishChanged(sender: UISwitch) {
        book.wish = sender.on
    }
    
    // MARK: - UITextFieldDelegate
    
    // テキスト編集開始時の処理
    func textFieldDidBeginEditing(textField: UITextField) {
        // 編集中UITextFieldの保持
        editingTextField = textField
    }
    
    
    // テキスト編集終了時の処理
    func textFieldDidEndEditing(textField: UITextField) {
        
        // 編集中UITextFieldの保持解除
        editingTextField = nil
        
        
        // テキストフィールドに応じた属性値をセット
        switch(textField) {
        case titleTextField:
            book.title = textField.text
        case authorTextField:
            book.author = textField.text
        case urlTextField:
            if let text = textField.text {
                book.url = NSURL(string: text)
            } else {
                book.url = nil
            }
        default:
            // do nothing
            break
        }
    }
    
    // returnキー押下時の処理
    func textFieldShouldReturn(textField: UITextField) -> Bool {
        // returnキーでキーボードを閉じる
        textField.resignFirstResponder()
        return true
    }
    
    // MARK: - UIImagePickerControllerDelegate
    
    // 画像選択時の処理
    func imagePickerController(picker: UIImagePickerController, didFinishPickingImage image: UIImage, editingInfo: [String : AnyObject]?) {
            
        // UIImageViewの更新
        photoImageView.image = image
        
        if let photo = book.photo {
            // photo関係がすでに存在しているとき→image属性を更新
            photo.image = UIImageJPEGRepresentation(image, 1.0)
        } else {
            // photo関係がnilのとき→新しいPhotoオブジェクトをコンテキストに挿入
            let newPhoto = NSEntityDescription.insertNewObjectForEntityForName("Photo",
                inManagedObjectContext: coreDataStack.context) as! Photo
            
            newPhoto.image = UIImageJPEGRepresentation(image, 1.0)
            book.photo = newPhoto
        }
        
        // UIImagePickerControllerを閉じる
        picker.dismissViewControllerAnimated(true, completion: nil)
    }
    
    // キャンセル時の処理
    func imagePickerControllerDidCancel(picker: UIImagePickerController) {
        // UIImagePickerControllerを閉じる
        picker.dismissViewControllerAnimated(true, completion: nil)
    }
    
    // MARK: - ShelfTableViewControllerDelegate
    func shelfTableViewController(controller: ShelfTableViewController,
        didSelectShelfWithObjectID objectID: NSManagedObjectID?) {
            
            // 不要になったShelfオブジェクトをフォールト化 --- (1)
            if let oldShelf = book.shelf {
                coreDataStack.context.refreshObject(oldShelf, mergeChanges: false)
            }
            
            if let objectID = objectID { // 本棚の選択あり
                // coreDataStack.context上にShelfオブジェクトを取り込む --- (2)
                let shelf = coreDataStack.context.objectWithID(objectID) as! Shelf
                book.shelf = shelf               // 関係の更新
                shelfNameLabel.text = shelf.name // UIの更新
            } else { // 本棚の選択なし
                book.shelf = nil           // 関係の更新
                shelfNameLabel.text = nil  // UIの更新
            }
    }
    

}
