//
//  BookTableViewController.swift
//  BookList
//
//  Created by Natsuko Nishikata on 2015/12/09.
//  Copyright © 2015年 Natsuko Nishikata. All rights reserved.
//

import UIKit
import CoreData

class BookTableViewController: UITableViewController, NSFetchedResultsControllerDelegate {
    
    let coreDataStack = CoreDataStack()
    
    // Fetched Results Controller
    lazy var fetchedResultsController:NSFetchedResultsController = {
        // フェッチ要求
        let fetchRequest = NSFetchRequest(entityName: "Book")
        
        // ソート順を指定（最低1つは必要）
        let dateSortDescriptor = NSSortDescriptor(key: "registeredDate", ascending: false)
        // titleのソートディスクリプタを追加
        let titleSortDescriptor = NSSortDescriptor(key: "title", ascending: true)
        // ソートがtitle、dateの順で行われるようにする
        fetchRequest.sortDescriptors = [titleSortDescriptor, dateSortDescriptor]
        
        // Fetched Results Controllerの初期化
        // sectionNameKeyPathを指定
        let frc = NSFetchedResultsController(fetchRequest: fetchRequest,
            managedObjectContext: self.coreDataStack.context,
            sectionNameKeyPath: "sectionIdentifier",
            cacheName: "Book")
        // デリゲートの設定
        frc.delegate = self
        
        return frc
    }()
    
    override func viewDidLoad() {
        super.viewDidLoad()
        
        navigationItem.leftBarButtonItem = editButtonItem()
        
        // Self-Sizingセルのための設定
        tableView.rowHeight = UITableViewAutomaticDimension
        tableView.estimatedRowHeight = 56.0
        
        NSFetchedResultsController.deleteCacheWithName("Book")
        
        // 永続ストアの追加
        userInteractionEnabled(false)
        coreDataStack.addPersistentStoreWithCompletionHandler() {
            // ストアの追加が完了
            dispatch_async(dispatch_get_main_queue()) {
                self.userInteractionEnabled(true)
                // フェッチ
                self.fetchBooks()
            }
        }
    }

    override func didReceiveMemoryWarning() {
        super.didReceiveMemoryWarning()
        // Dispose of any resources that can be recreated.
    }
    
    override func setEditing(editing: Bool, animated: Bool) {
        super.setEditing(editing, animated: animated)
        navigationItem.rightBarButtonItem?.enabled = !editing
    }


    // MARK: - Table view data source
    
    // テーブルビューのセクション数
    override func numberOfSectionsInTableView(tableView: UITableView) -> Int {
        return fetchedResultsController.sections?.count ?? 0
    }

    // セクションsectionに含まれる行数（必須）
    override func tableView(tableView: UITableView, numberOfRowsInSection section: Int) -> Int {
        let sectionInfo = fetchedResultsController.sections?[section]
        return sectionInfo?.numberOfObjects ?? 0
    }
    
    // セルの設定（必須）
    override func tableView(tableView: UITableView, cellForRowAtIndexPath indexPath: NSIndexPath) -> UITableViewCell {
            
        // "BookCell"はStoryboardで設定済み
        let cell = tableView.dequeueReusableCellWithIdentifier("BookCell",
            forIndexPath: indexPath) as! BookTableViewCell
        
        // セルを設定
        let book = fetchedResultsController.objectAtIndexPath(indexPath) as! Book

        cell.configureWithBook(book)
        return cell
    }

    // 編集可能かどうかを示す
    override func tableView(tableView: UITableView, canEditRowAtIndexPath indexPath: NSIndexPath) -> Bool {
        return true
    }
    
    // 削除時の処理を実装する
    override func tableView(tableView: UITableView, commitEditingStyle editingStyle: UITableViewCellEditingStyle, forRowAtIndexPath indexPath: NSIndexPath) {

        if editingStyle == .Delete {
            
            // 削除対象となるBookオブジェクト
            let book = fetchedResultsController.objectAtIndexPath(indexPath) as! Book
            // 管理オブジェクトコンテキストから削除
            coreDataStack.context.deleteObject(book)
            try! coreDataStack.saveContext()
        }
    }
    
    // セクション名を表示
    override func tableView(tableView: UITableView, titleForHeaderInSection section: Int) -> String? {
        let sectionInfo = fetchedResultsController.sections![section]
        return sectionInfo.name
    }
    
    // セクションインデックスタイトルに対応するセクションを返す
    override func tableView(tableView: UITableView, sectionForSectionIndexTitle title: String, atIndex index: Int) -> Int {
        return fetchedResultsController.sectionForSectionIndexTitle(title, atIndex: index)
    }
    
    // セクションインデックスタイトルを返す
    override
    func sectionIndexTitlesForTableView(tableView: UITableView) -> [String]? {
        return fetchedResultsController.sectionIndexTitles
    }
    
    // MARK: - Fetched Results Controller
    // フェッチ結果の変更開始
    func controllerWillChangeContent(controller: NSFetchedResultsController) {
        // テーブルビューの更新を開始
        tableView.beginUpdates()
    }
    
    // フェッチ結果の変更終了
    func controllerDidChangeContent(controller: NSFetchedResultsController) {
        // テーブルビューの更新を終了
        tableView.endUpdates()
    }
    
    // セクション情報の変更
    func controller(controller: NSFetchedResultsController, didChangeSection sectionInfo: NSFetchedResultsSectionInfo, atIndex sectionIndex: Int, forChangeType type: NSFetchedResultsChangeType) {
        let sections = NSIndexSet(index: sectionIndex)
        switch type {
        case .Insert:
            // セクションを挿入
            tableView.insertSections(sections, withRowAnimation: .Fade)
        case .Delete:
            // セクションを削除
            tableView.deleteSections(sections, withRowAnimation: .Fade)
        default:
            return
        }
    }
    
    // オブジェクトの変更
    func controller(controller: NSFetchedResultsController, didChangeObject anObject: AnyObject, atIndexPath indexPath: NSIndexPath?, forChangeType type: NSFetchedResultsChangeType, newIndexPath: NSIndexPath?) {
        switch type {
        case .Insert:
            // 行を挿入
            tableView.insertRowsAtIndexPaths([newIndexPath!],
                withRowAnimation: .Fade)
        case .Delete:
            // 行を削除
            tableView.deleteRowsAtIndexPaths([indexPath!],
                withRowAnimation: .Fade)
        case .Update:
            // セルのコンテンツを更新
            let cell =
            tableView.cellForRowAtIndexPath(indexPath!) as! BookTableViewCell
            let book = anObject as! Book
            cell.configureWithBook(book)
        case .Move:
            // セルを移動
            tableView.deleteRowsAtIndexPaths([indexPath!],
                withRowAnimation: .Fade)
            tableView.insertRowsAtIndexPaths([newIndexPath!],
                withRowAnimation: .Fade)
        }
    }
    
    // インデックスタイトルを指定
    func controller(controller: NSFetchedResultsController, sectionIndexTitleForSectionName sectionName: String) -> String? {
        return sectionName
    }

    // MARK: - Navigation

    // In a storyboard-based application, you will often want to do a little preparation before navigation
    override func prepareForSegue(segue: UIStoryboardSegue, sender: AnyObject?) {
        
        if segue.identifier == "EditBook" {
            var book:Book // 編集対象となるBookオブジェクト
            if let cell = sender as? UITableViewCell {
                // セルタップによるセグエ実行
                let indexPath = tableView.indexPathForCell(cell)!
                //  データソースから編集対象となるオブジェクトを取得
                book = fetchedResultsController.objectAtIndexPath(indexPath) as! Book
            } else {
                // 「＋」ボタンからのセグエ実行
                // コンテキストに挿入されたBookオブジェクトが編集対象
                book = coreDataStack.context.insertedObjects.first as! Book
            }
            // セグエの接続先ビューコントローラはナビゲーションコントローラなので
            // そこから編集用ビューコントローラをたどる
            let nvc = segue.destinationViewController as! UINavigationController
            let editVC = nvc.topViewController as! BookEditTableViewController
            editVC.book = book // 編集対象となるBookオブジェクトをセット
            editVC.coreDataStack = coreDataStack
        }
    }


    // MARK: - Actions
    
    @IBAction func addBook(sender: UIBarButtonItem) {
        // coreDataStack.contextに新しい管理オブジェクトnewBookを追加
        NSEntityDescription.insertNewObjectForEntityForName("Book",
            inManagedObjectContext: coreDataStack.context)
        
        // セグエを実行して編集画面へ遷移
        self.performSegueWithIdentifier("EditBook", sender: nil)
    }
    
    @IBAction func segmentChanged(sender: UISegmentedControl) {
        // index 1のセグメントが選択されているときは、欲しい本リストを作る
        let wish = sender.selectedSegmentIndex == 1
        if wish {
            // wishがtrueのBookだけをフェッチするための条件指定
            let predicate = NSPredicate(format: "wish == true")
            fetchedResultsController.fetchRequest.predicate = predicate
        } else {
            // 条件指定を解除
            fetchedResultsController.fetchRequest.predicate = nil
        }
        NSFetchedResultsController.deleteCacheWithName("Book")
        fetchBooks()
    }
    
    // MARK: - Core Data
    private func fetchBooks() {
        do {
            // Fetched Results Controllerでフェッチを実行
            try fetchedResultsController.performFetch()
        } catch let error as NSError {
            fatalError("\(error)")
        }
        
        // テーブルビューをリロード
        tableView.reloadData()
    }
    
    // MARK: - private methods
    private func userInteractionEnabled(enabled:Bool) {
        self.navigationItem.rightBarButtonItem?.enabled = enabled
        self.navigationItem.leftBarButtonItem?.enabled = enabled
        (self.navigationItem.titleView as! UISegmentedControl).enabled = enabled
    }
}

class BookTableViewCell: UITableViewCell {
    @IBOutlet weak var titleLabel: UILabel!
    @IBOutlet weak var authorLabel: UILabel!
    @IBOutlet weak var wishLabel: UILabel!
    @IBOutlet weak var dateLabel: UILabel!
    
    lazy var dateFormatter: NSDateFormatter = {
        let dateFormatter = NSDateFormatter()
        dateFormatter.dateStyle = .ShortStyle
        dateFormatter.timeStyle = .MediumStyle
        return dateFormatter
    }()
    
    func configureWithBook(book:Book) {
        titleLabel.text = book.title
        authorLabel.text = book.author
        wishLabel.hidden = !(book.wish!.boolValue)
        dateLabel.text =
            dateFormatter.stringFromDate(book.registeredDate!)
    }
    
}
