//  Created by Natsuko Nishikata
//  Copyright © 2016年 Natsuko Nishikata. All rights reserved.

//: Playground - noun: a place where people can play

import CoreData

/********************************************************
 管理オブジェクトモデルの作成
********************************************************/

// 管理オブジェクトモデルの初期化
let model = NSManagedObjectModel()

// エンティティ記述子の初期化
let itemEntity = NSEntityDescription()
itemEntity.name = "Item"
let shopEntity = NSEntityDescription()
shopEntity.name = "Shop"

// Itemに追加するプロパティ
// title属性
let titleAttribute = NSAttributeDescription()
titleAttribute.name = "title" // 名前
titleAttribute.attributeType = .StringAttributeType // 型
titleAttribute.optional = false // オプショナル性

// price属性
let priceAttribute = NSAttributeDescription()
priceAttribute.name = "price"
priceAttribute.attributeType = .Integer32AttributeType
priceAttribute.optional = true

// shop関係
let shopRelationship = NSRelationshipDescription()
shopRelationship.name = "shop" // 名前
shopRelationship.destinationEntity = shopEntity // 関係先
shopRelationship.deleteRule = .NullifyDeleteRule // 削除ルール
shopRelationship.minCount = 0 // 最小数
shopRelationship.maxCount = 1 // 最大数（対１関係）

// Shopに追加するプロパティ
// name属性
let nameAttribute = NSAttributeDescription()
nameAttribute.name = "name"
nameAttribute.attributeType = .StringAttributeType
nameAttribute.optional = false

// items関係
let itemsRelationship = NSRelationshipDescription()
itemsRelationship.name = "items"
itemsRelationship.destinationEntity = itemEntity
itemsRelationship.deleteRule = .NullifyDeleteRule
itemsRelationship.minCount = 0
itemsRelationship.maxCount = 0 // 無制限（対多関係）

// 逆関係の設定
shopRelationship.inverseRelationship = itemsRelationship
itemsRelationship.inverseRelationship = shopRelationship

// プロパティの追加
itemEntity.properties = [titleAttribute, priceAttribute, shopRelationship]
shopEntity.properties = [nameAttribute, itemsRelationship]

// エンティティの追加
model.entities = [itemEntity, shopEntity]

/********************************************************
 Core Dataスタックの初期化（SQLiteストア）
********************************************************/

// 永続ストアコーディネータの初期化
let coordinator = NSPersistentStoreCoordinator(managedObjectModel: model)

// 永続ストアURLの初期化
let fileManager = NSFileManager.defaultManager()
let urls = fileManager.URLsForDirectory(.DocumentDirectory,
    inDomains:.UserDomainMask)
let dirUrl = urls[urls.count - 1]

let storeURL = dirUrl.URLByAppendingPathComponent("PGCoreDataTest.sqlite")

do {
    // 永続ストアをコーディネータに追加
    try coordinator.addPersistentStoreWithType(NSSQLiteStoreType,
        configuration: nil,
        URL: storeURL,
        options: nil)
} catch {
    print("\(error)")
}

// 管理オブジェクトコンテキストの初期化
let moc = NSManagedObjectContext(concurrencyType: .MainQueueConcurrencyType)
moc.persistentStoreCoordinator = coordinator

/********************************************************
 初期データの準備
********************************************************/

// 永続データ内のItemの数を確認
// フェッチ要求の生成
let fetchRequest = NSFetchRequest(entityName: "Item")
fetchRequest.resultType = .CountResultType // フェッチ結果の型を指定

// Item数の確認
let count = try! moc.executeFetchRequest(fetchRequest)[0] as! Int

// 初期データの作成
if count == 0 {
    
    var apple = NSEntityDescription.insertNewObjectForEntityForName("Item",
        inManagedObjectContext: moc)
    apple.setValue("Apple", forKey: "title")
    apple.setValue(100, forKey: "price")
    
    var milk = NSEntityDescription.insertNewObjectForEntityForName("Item",
        inManagedObjectContext: moc)
    milk.setValue("Milk", forKey: "title")
    milk.setValue(200, forKey: "price")
    
    var book = NSEntityDescription.insertNewObjectForEntityForName("Item",
        inManagedObjectContext: moc)
    book.setValue("Core Data Programming Book", forKey: "title")
    book.setValue(3500, forKey: "price")
    
    var supermarket =
    NSEntityDescription.insertNewObjectForEntityForName("Shop",
        inManagedObjectContext: moc)
    supermarket.setValue("Supermarket", forKey: "name")
    
    var bookstore =
    NSEntityDescription.insertNewObjectForEntityForName("Shop",
        inManagedObjectContext: moc)
    bookstore.setValue("Book Store", forKey: "name")
    
    apple.setValue(supermarket, forKey: "shop")
    milk.setValue(supermarket, forKey: "shop")
    book.setValue(bookstore, forKey: "shop")
    
    try! moc.save()
}

/********************************************************
 フェッチ・管理オブジェクトの編集
********************************************************/

// フェッチ要求の型を管理オブジェクトに変更
fetchRequest.resultType = .ManagedObjectResultType

// ソートの設定
let priceSort = NSSortDescriptor(key: "price", ascending: true)
let shopSort = NSSortDescriptor(key: "shop.name", ascending: true)
fetchRequest.sortDescriptors = [shopSort, priceSort]

// 条件の設定
let predicate = NSPredicate(format: "price >= 100")
fetchRequest.predicate = predicate

// フェッチ
let items = try! moc.executeFetchRequest(fetchRequest) as! [NSManagedObject]

/********************************************************
 NSExpressionDescriptionによる値の検索
********************************************************/

// 結果型を辞書に変更
fetchRequest.resultType = .DictionaryResultType

// 検索のためのNSExpressionを作成
let keyPathExpression = NSExpression(forKeyPath: "price")
let maxPriceExpression =
NSExpression(forFunction: "max:", arguments: [keyPathExpression])
let minPriceExpression =
NSExpression(forFunction: "min:", arguments: [keyPathExpression])
let avgPriceExpression =
NSExpression(forFunction: "average:", arguments: [keyPathExpression])

// NSExpressionDescriptionを作成
let maxExpDescription = NSExpressionDescription()
maxExpDescription.name = "maxPrice"
maxExpDescription.expression = maxPriceExpression
maxExpDescription.expressionResultType = .Integer32AttributeType

let minExpDescription = NSExpressionDescription()
minExpDescription.name = "minPrice"
minExpDescription.expression = minPriceExpression
minExpDescription.expressionResultType = .Integer32AttributeType

let avgExpDescription = NSExpressionDescription()
avgExpDescription.name = "avgPrice"
avgExpDescription.expression = avgPriceExpression
avgExpDescription.expressionResultType = .Integer32AttributeType

// フェッチすべきプロパティを式プロパティに限定する
fetchRequest.propertiesToFetch =
    [maxExpDescription, minExpDescription, avgExpDescription]

// 結果を取得する
let expResult = try! moc.executeFetchRequest(fetchRequest)


